==> sentry-23.4.0-0-py38-none-any.whl: sentry/models/counter.py <==
import random

from django.conf import settings
from django.db import connections, transaction
from django.db.models.signals import post_migrate

from sentry import options
from sentry.db.models import (
    BoundedBigIntegerField,
    FlexibleForeignKey,
    Model,
    get_model_if_available,
    region_silo_only_model,
    sane_repr,
)
from sentry.db.postgres.roles import in_test_psql_role_override


@region_silo_only_model
class Counter(Model):
    __include_in_export__ = True

    project = FlexibleForeignKey("sentry.Project", unique=True)
    value = BoundedBigIntegerField()

    __repr__ = sane_repr("project")

    class Meta:
        app_label = "sentry"
        db_table = "sentry_projectcounter"

    @classmethod
    def increment(cls, project, delta=1):
        """Increments a counter.  This can never decrement."""
        return increment_project_counter(project, delta)


def increment_project_counter(project, delta=1, using="default"):
    """This method primarily exists so that south code can use it."""
    if delta <= 0:
        raise ValueError("There is only one way, and that's up.")

    sample_rate = options.get("store.projectcounter-modern-upsert-sample-rate")

    modern_upsert = sample_rate and random.random() <= sample_rate

    # To prevent the statement_timeout leaking into the session we need to use
    # set local which can be used only within a transaction
    with transaction.atomic(using=using):
        cur = connections[using].cursor()
        try:
            statement_timeout = None
            if settings.SENTRY_PROJECT_COUNTER_STATEMENT_TIMEOUT:
                # WARNING: This is not a proper fix and should be removed once
                #          we have better way of generating next_short_id.
                cur.execute("show statement_timeout")
                statement_timeout = cur.fetchone()[0]
                cur.execute(
                    "set local statement_timeout = %s",
                    [settings.SENTRY_PROJECT_COUNTER_STATEMENT_TIMEOUT],
                )

            if modern_upsert:
                # Our postgres wrapper thing does not allow for named arguments
                cur.execute(
                    "insert into sentry_projectcounter (project_id, value) "
                    "values (%s, %s) "
                    "on conflict (project_id) do update "
                    "set value = sentry_projectcounter.value + %s "
                    "returning value",
                    [project.id, delta, delta],
                )
            else:
                cur.execute(
                    "select sentry_increment_project_counter(%s, %s)",
                    [project.id, delta],
                )

            project_counter = cur.fetchone()[0]

            if statement_timeout is not None:
                cur.execute(
                    "set local statement_timeout = %s",
                    [statement_timeout],
                )

            return project_counter

        finally:
            cur.close()


# this must be idempotent because it seems to execute twice
# (at least during test runs)
def create_counter_function(app_config, using, **kwargs):
    if app_config and app_config.name != "sentry":
        return

    if not get_model_if_available(app_config, "Counter"):
        return

    with in_test_psql_role_override("postgres", using), connections[using].cursor() as cursor:
        cursor.execute(
            """
            create or replace function sentry_increment_project_counter(
                project bigint, delta int) returns int as $$
            declare
            new_val int;
            begin
            loop
                update sentry_projectcounter set value = value + delta
                where project_id = project
                returning value into new_val;
                if found then
                return new_val;
                end if;
                begin
                insert into sentry_projectcounter(project_id, value)
                    values (project, delta)
                    returning value into new_val;
                return new_val;
                exception when unique_violation then
                end;
            end loop;
            end
            $$ language plpgsql;
        """
        )


post_migrate.connect(create_counter_function, dispatch_uid="create_counter_function", weak=False)

==> sentry-23.7.0-0-py38-none-any.whl: sentry/models/counter.py <==
import random

from django.conf import settings
from django.db import connections, transaction
from django.db.models.signals import post_migrate

from sentry import options
from sentry.db.models import (
    BoundedBigIntegerField,
    FlexibleForeignKey,
    Model,
    get_model_if_available,
    region_silo_only_model,
    sane_repr,
)
from sentry.silo import SiloMode, unguarded_write


@region_silo_only_model
class Counter(Model):
    __include_in_export__ = True

    project = FlexibleForeignKey("sentry.Project", unique=True)
    value = BoundedBigIntegerField()

    __repr__ = sane_repr("project")

    class Meta:
        app_label = "sentry"
        db_table = "sentry_projectcounter"

    @classmethod
    def increment(cls, project, delta=1):
        """Increments a counter.  This can never decrement."""
        return increment_project_counter(project, delta)


def increment_project_counter(project, delta=1, using="default"):
    """This method primarily exists so that south code can use it."""
    if delta <= 0:
        raise ValueError("There is only one way, and that's up.")

    sample_rate = options.get("store.projectcounter-modern-upsert-sample-rate")

    modern_upsert = sample_rate and random.random() <= sample_rate

    # To prevent the statement_timeout leaking into the session we need to use
    # set local which can be used only within a transaction
    with transaction.atomic(using=using):
        cur = connections[using].cursor()
        try:
            statement_timeout = None
            if settings.SENTRY_PROJECT_COUNTER_STATEMENT_TIMEOUT:
                # WARNING: This is not a proper fix and should be removed once
                #          we have better way of generating next_short_id.
                cur.execute("show statement_timeout")
                statement_timeout = cur.fetchone()[0]
                cur.execute(
                    "set local statement_timeout = %s",
                    [settings.SENTRY_PROJECT_COUNTER_STATEMENT_TIMEOUT],
                )

            if modern_upsert:
                # Our postgres wrapper thing does not allow for named arguments
                cur.execute(
                    "insert into sentry_projectcounter (project_id, value) "
                    "values (%s, %s) "
                    "on conflict (project_id) do update "
                    "set value = sentry_projectcounter.value + %s "
                    "returning value",
                    [project.id, delta, delta],
                )
            else:
                cur.execute(
                    "select sentry_increment_project_counter(%s, %s)",
                    [project.id, delta],
                )

            project_counter = cur.fetchone()[0]

            if statement_timeout is not None:
                cur.execute(
                    "set local statement_timeout = %s",
                    [statement_timeout],
                )

            return project_counter

        finally:
            cur.close()


# this must be idempotent because it seems to execute twice
# (at least during test runs)
def create_counter_function(app_config, using, **kwargs):
    if app_config and app_config.name != "sentry":
        return

    if not get_model_if_available(app_config, "Counter"):
        return

    if SiloMode.get_current_mode() == SiloMode.CONTROL:
        return

    with unguarded_write(using), connections[using].cursor() as cursor:
        cursor.execute(
            """
            create or replace function sentry_increment_project_counter(
                project bigint, delta int) returns int as $$
            declare
            new_val int;
            begin
            loop
                update sentry_projectcounter set value = value + delta
                where project_id = project
                returning value into new_val;
                if found then
                return new_val;
                end if;
                begin
                insert into sentry_projectcounter(project_id, value)
                    values (project, delta)
                    returning value into new_val;
                return new_val;
                exception when unique_violation then
                end;
            end loop;
            end
            $$ language plpgsql;
        """
        )


post_migrate.connect(create_counter_function, dispatch_uid="create_counter_function", weak=False)
